unit uescenarios;
{
rch@20130914
Un escenario está definido por un conjunto de capas activas.
Las capas son números enteros
}


interface

uses
  Classes, SysUtils, xMatDefs, uCosa, uCosaConNombre;

type

  { TEscenario_rec }

  TEscenario_rec = class( TCosaConNombre )
    capasActivas: TDAOfNInt;
    activa: boolean;
    descripcion: string;
    run_opt, run_sim: boolean;
    constructor Create( nombre: string );
    constructor Create_ReadFromText(f: TArchiTexto ); override;
    procedure WriteToText(f: TArchiTexto); override;
    procedure Free;
    procedure AddCapa( capa: integer );
    procedure DelCapa( capa: integer );
    function tieneCapa( capa: integer): boolean;

    // Chequea si la Cosa está en una capa de las activas
    // del escenario y retonra TRUE en caso afirmativo.
    function Participa( Cosa: TCosa ): boolean;


    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;


  end;

  { TListaEscenarios }

  TListaEscenarios = class(TListaDeCosasConNombre)
    constructor Create(nombre: string);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText( f: TArchiTexto); override;

// chequea si el escenario ya no está en la lista y lo agrega.
// retorna el puntero al Rec agregado o NIL si ya estaba y no fue agregado.
    function AppendEscenario( nombreEscenario: string ): TEscenario_rec;

    function lista_activas: TStrings;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; override;


end;

procedure AlInicio;
procedure AlFinal;

implementation


constructor TEscenario_rec.Create( nombre: string );
begin
  inherited Create(0, nombre );
  setlength( capasActivas, 1 );
  capasActivas[0]:= 0;
  activa:= true;
  run_opt:= true;
  run_sim:= true;
end;



constructor TEscenario_rec.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText( f );
  f.rd( 'capasActivas', capasActivas );
  f.rd( 'activa', activa );
  f.rd( 'descripcion', descripcion );
  f.rd( 'run_opt', run_opt );
  f.rd( 'run_sim', run_sim );
end;

procedure TEscenario_rec.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText( f );
  f.wr( 'capasActivas', capasActivas );
  f.wr( 'activa', activa );
  f.wr( 'descripcion', descripcion );
  f.wr( 'run_opt', run_opt );
  f.wr( 'run_sim', run_sim );
end;

procedure TEscenario_rec.Free;
begin
  setlength( capasActivas, 0 );
  inherited Free;
end;

procedure TEscenario_rec.AddCapa( capa: integer );
begin
  addToArray( capasActivas, capa );
end;

procedure TEscenario_rec.DelCapa( capa: integer );
begin
  delFromArray( capasActivas, capa );
end;

function TEscenario_rec.tieneCapa( capa: integer): boolean;
begin
  result:= kInArray( capasActivas, capa ) >= 0;
end;


function TEscenario_rec.Participa( Cosa: TCosa ): boolean;
begin
  result:= kInArray( capasActivas, Cosa.capa ) >= 0;
end;


class function TEscenario_rec.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa;
  xVersion: Integer): TDataColumnListOfCosa;
begin

  Result:=TDataColumnListOfCosa.Create(TEscenario_rec, xVersion);

  Result.AddList(inherited CreateDataColumnList(xClaseDeCosa, xVersion));

  Result.AddDataColumn('activa',       TEscenario_rec(nil).activa,       TDataTypeBooleanOfCosa,     ['Activo', ''],                     113, -1);
  Result.AddDataColumn('descripcion',  TEscenario_rec(nil).descripcion,  TDataTypeStringOfCosa,      ['Descripción', ''],                113, -1);
  Result.AddDataColumn('capasActivas', TEscenario_rec(nil).capasActivas, TDataTypeDAOfIntegerOfCosa, ['Capas Activas', ''],              113, -1);
  Result.AddDataColumn('run_opt',      TEscenario_rec(nil).run_opt,      TDataTypeBooleanOfCosa,     ['Ejecutar en el Optimizador', ''], 113, -1);
  Result.AddDataColumn('run_sim',      TEscenario_rec(nil).run_sim,      TDataTypeBooleanOfCosa,     ['Ejecutar en el Simulador', ''],   113, -1);

end;


constructor TListaEscenarios.Create(nombre: string);
var
 ae: TEscenario_rec;
begin
  inherited Create(0, nombre );
  // por defecto ponemos escenario base con capa cero
  ae:= TEscenario_rec.Create('Base');
  add( ae );
end;

constructor TListaEscenarios.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText( f );
end;

procedure TListaEscenarios.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText( f );
end;




function TListaEscenarios.lista_activas: TStrings;

var
 k: integer;
 rec: TEscenario_rec;
 res: TStringList;

begin
  res:= TStringList.Create;
  for k:= 0 to count -1 do
  begin
    rec:= items[k] as TEscenario_rec;
    if rec.activa then
       res.Add( rec.nombre );
  end;
  result:= res;
end;


class function TListaEscenarios.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list:TDataColumnListOfCosa;
begin
  list:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result:=list;
end;


function TListaEscenarios.AppendEscenario( nombreEscenario: string ): TEscenario_rec;
var
 rec: TEscenario_rec;
begin
  if Find( nombreEscenario ) = nil then
  begin
    rec:= TEscenario_rec.Create( nombreEscenario );
    add( rec );
    result:= rec;
  end
  else
    result:= nil;
end;



(*********************************)
procedure AlInicio;
begin
  registrarClaseDeCosa(TEscenario_rec.ClassName, TEscenario_rec );
  registrarClaseDeCosa(TListaEscenarios.ClassName, TListaEscenarios );
end;

procedure AlFinal;
begin
end;

end.

