unit uFuenteSelector;

interface

uses
  uFuentesAleatorias, ucosa, uFichasLPD, xMatDefs, uFechas;

resourcestring
  rsFichaDe = 'Ficha de';
  rsFuenteSelector = 'Fuente Selector';

type

(*+doc TFichaFuenteSelector
Ficha de parámetros dinámicos de las fuentes Selector
Permite especificar fuentes (y bornes) de entrada
-doc*)

  { TFichaFuenteSelector }

  TFichaFuenteSelector = class(TFichaLPD)
  public
    fuenteA, fuenteB: TFuenteAleatoria;
    fuenteC, fuenteD: TFuenteAleatoria;
    borneA, borneB: string;
    borneC, borneD: string;

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      fuenteA, fuenteB: TFuenteAleatoria; fuenteC, fuenteD: TFuenteAleatoria;
      borneA, borneB: string; borneC, borneD: string);
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    class function DescClase: string; override;
    function infoAd_: string; override;

    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer = -2): TDataColumnListOfCosa; virtual;

  end;

(*+doc TFuenteSelector
Genera sus salidas a partir de los valores de 4 fuentes aleatorias
fuenteA , fuenteB, fuenteC y fuente D.
Si ( A > B ) el resultado es C sino es D
-doc*)
  TFuenteSelector = class(TFuenteAleatoriaConFichas)
  private
    //Indices de los bornes de las fuentes de entrada. Deben ser
    //actualizados al cambiar la ficha de parametros dinámicos
    iBorneA, iBorneB: integer;
    iBorneC, iBorneD: integer;
  public
    pa: TFichaFuenteSelector;

    constructor Create(capa: integer; nombre: string; lpd: TFichasLPD);
    class function TipoFichaFuente: TClaseDeFichaLPD; override;
    //function InfoAd : String; override;
    class function DescClase: string; override;
    procedure RegistrarParametrosDinamicos; override;

    procedure PrepararPaso_ps; override;
    procedure SortearEntradaRB(var aRB: NReal); override;
    procedure ValorEsperadoEntradaRB(var aRB: Nreal); override;
    function referenciaFuente(fuente: TFuenteAleatoria): boolean; override;

  end;

procedure procCambioFichaSelector(fuente: TCosa);
procedure AlInicio;
procedure AlFinal;

implementation

//-----------------------------------
// Métodos de TFichaFuenteSelector
//===================================

constructor TFichaFuenteSelector.Create(capa: integer; fecha: TFecha;
  periodicidad: TPeriodicidad; fuenteA, fuenteB: TFuenteAleatoria;
  fuenteC, fuenteD: TFuenteAleatoria; borneA, borneB: string; borneC, borneD: string);
begin
  inherited Create(capa, fecha, periodicidad);
  self.fuenteA := fuenteA;
  self.fuenteB := fuenteB;
  self.fuenteC := fuenteC;
  self.fuenteD := fuenteD;
  self.borneA := borneA;
  self.borneB := borneB;
  self.borneC := borneC;
  self.borneD := borneD;
end;

constructor TFichaFuenteSelector.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('fuenteA', TCosa(fuenteA), self);
  f.rdReferencia('fuenteB', TCosa(fuenteB), self);
  f.rdReferencia('fuenteC', TCosa(fuenteC), self);
  f.rdReferencia('fuenteD', TCosa(fuenteD), self);
  f.rd('borneA', borneA);
  f.rd('borneB', borneB);
  f.rd('borneC', borneC);
  f.rd('borneD', borneD);
  f.EjecutarLectura;
end;

procedure TFichaFuenteSelector.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('fuenteA', TCosa(fuenteA));
  f.wrReferencia('fuenteB', TCosa(fuenteB));
  f.wrReferencia('fuenteC', TCosa(fuenteC));
  f.wrReferencia('fuenteD', TCosa(fuenteD));
  f.wr('borneA', borneA);
  f.wr('borneB', borneB);
  f.wr('borneC', borneC);
  f.wr('borneD', borneD);
end;

class function TFichaFuenteSelector.DescClase: string;
begin
  Result := rsFichaDe + ' ' + rsFuenteSelector;
end;

function TFichaFuenteSelector.infoAd_: string;
begin
  Result := 'fuenteA= ' + fuenteA.nombre + ', ' + 'borneA= ' +
    borneA + ', ' + 'fuenteB= ' + fuenteB.nombre + ', ' + 'borneB= ' + borneB;
end;

class function TFichaFuenteSelector.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
var
  list: TDataColumnListOfCosa;
begin
  //list:= TDataColumnListOfCosa.Create(TFichaFuenteSelector, xVersion);
  //list.AddList(inherited CreateDataColumnList(xVersion));
  list:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  list.AddDataColumn('fuenteA', TFichaFuenteSelector(nil).fuenteA, TDataTypeReferenciaOfCosa, 113, -1);
  list.AddDataColumn('fuenteB', TFichaFuenteSelector(nil).fuenteB, TDataTypeReferenciaOfCosa, 113, -1);
  list.AddDataColumn('fuenteC', TFichaFuenteSelector(nil).fuenteC, TDataTypeReferenciaOfCosa, 113, -1);
  list.AddDataColumn('fuenteD', TFichaFuenteSelector(nil).fuenteD, TDataTypeReferenciaOfCosa, 113, -1);
  list.AddDataColumn('borneA', TFichaFuenteSelector(nil).borneA, TDataTypeStringOfCosa, 113, -1);
  list.AddDataColumn('borneB', TFichaFuenteSelector(nil).borneB, TDataTypeStringOfCosa, 113, -1);
  list.AddDataColumn('borneC', TFichaFuenteSelector(nil).borneC, TDataTypeStringOfCosa, 113, -1);
  list.AddDataColumn('borneD', TFichaFuenteSelector(nil).borneD, TDataTypeStringOfCosa, 113, -1);
  Result := list;
end;

//------------------------------
// Métodos de TFuenteSelector
//==============================

constructor TFuenteSelector.Create(capa: integer; nombre: string; lpd: TFichasLPD);
begin
  inherited Create(capa, nombre, 0, True, lpd);
end;

class function TFuenteSelector.TipoFichaFuente: TClaseDeFichaLPD;
begin
  Result := TFichaFuenteSelector;
end;

{function TFuenteSelector.InfoAd : String;
begin
  result:= 'Selector';
end;}

class function TFuenteSelector.DescClase: string;
begin
  Result := rsFuenteSelector;
end;

procedure TFuenteSelector.RegistrarParametrosDinamicos;
begin
  //Se llama solo despues de preparar memoria que ya expande las fichas
  //no hace falta volver a hacerlo y es un proceso caro si hay periodicidad
  //   lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pa,
    nil, procCambioFichaSelector);
end;

procedure TFuenteSelector.PrepararPaso_ps;
begin
  if pa.fuenteA.Bornera[iBorneA] > pa.fuenteB.Bornera[iBorneB] then
    Bornera[0] := pa.fuenteC.Bornera[iBorneC]
  else
    Bornera[0] := pa.fuenteD.Bornera[iBorneD];
end;

procedure TFuenteSelector.SortearEntradaRB(var aRB: NReal);
begin
  aRB := 0;
end;

procedure TFuenteSelector.ValorEsperadoEntradaRB(var aRB: Nreal);
begin
  aRB := 0;
end;


function TFuenteSelector.referenciaFuente(fuente: TFuenteAleatoria): boolean;
var
  i: integer;
  res: boolean;
begin
  res := False;
  for i := 0 to lpd.Count - 1 do
  begin
    if (fuente = TFichaFuenteSelector(lpd[i]).fuenteA) or
      (fuente = TFichaFuenteSelector(lpd[i]).fuenteB) or
      (fuente = TFichaFuenteSelector(lpd[i]).fuenteC) or
      (fuente = TFichaFuenteSelector(lpd[i]).fuenteD) then
    begin
      res := True;
      break;
    end;
  end;
  Result := res;
end;

procedure procCambioFichaSelector(fuente: TCosa);
begin
  cambioFichaPDFuenteAleatoria(fuente);
  TFuenteSelector(fuente).iBorneA :=
    TFuenteSelector(fuente).pa.fuenteA.IdBorne(TFuenteSelector(fuente).pa.borneA);
  TFuenteSelector(fuente).iBorneB :=
    TFuenteSelector(fuente).pa.fuenteB.IdBorne(TFuenteSelector(fuente).pa.borneB);
  TFuenteSelector(fuente).iBorneC :=
    TFuenteSelector(fuente).pa.fuenteC.IdBorne(TFuenteSelector(fuente).pa.borneC);
  TFuenteSelector(fuente).iBorneD :=
    TFuenteSelector(fuente).pa.fuenteD.IdBorne(TFuenteSelector(fuente).pa.borneD);
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TFuenteSelector.ClassName, TFuenteSelector);
  registrarClaseDeCosa(TFichaFuenteSelector.ClassName, TFichaFuenteSelector);
end;

procedure AlFinal;
begin
end;

end.
